/////////////Librerías de babylon usadas en este proyecto////////////
// <reference path="./Babylon js/babylon.js" />
// <reference path="./Babylon js/babylon.d.ts" />
// <reference path="./Babylon js/babylon.gui.d.ts" />
// <reference path="./Babylon js/babylon.inspector.bundle.js" />
// <reference path="./Babylon js/babylonjs.loaders.min.js" />
// <reference path="./Babylon js/cannon.js" />
// <reference path="./Babylon js/ammo.js" />
////////////////////////////////////////////////////////////////////

class SceneOne 
{
    private _scene: BABYLON.Scene;
    
    constructor(engine:BABYLON.Engine,canvas:HTMLCanvasElement)
    {
        this._scene = this.CreateScene(engine);
        this.CreateCamera(canvas,this._scene);
        this.CreateLighting(this._scene)
        
        let multiSphere:MultipleSphere = new MultipleSphere(this._scene);//create multiple sphere
        
        this.ResizeWindow(engine);
    }
    private CreateScene(engine:BABYLON.Engine):BABYLON.Scene
    {
        let scene:BABYLON.Scene = new BABYLON.Scene(engine);
        return scene;
    }

    private CreateCamera(canvas:HTMLElement,scene:BABYLON.Scene)
    {
        let camera = new BABYLON.ArcRotateCamera("Camera", Math.PI / 2, Math.PI / 2, 2, new BABYLON.Vector3(0,0,5), scene);
        camera.attachControl(canvas, true);
    }

    private CreateLighting(scene:BABYLON.Scene):BABYLON.DirectionalLight
    {
        let light:BABYLON.DirectionalLight = new BABYLON.DirectionalLight("light3", new BABYLON.Vector3(0, 1, 0), scene);
        return light;
    }

    private ResizeWindow(engine:BABYLON.Engine)
    {
        window.addEventListener("resize", function () { //agrego el evento para que constantemente se acutalice el navegador
            engine.resize();
        });
    }
    public get scene(): BABYLON.Scene {
        return this._scene;
    }
    public set scene(value: BABYLON.Scene) {
            this._scene = value;
    }

}

export class Sphere {
    private _sphere: BABYLON.Mesh;
    private _position: BABYLON.Vector3;
    

    constructor(scene:BABYLON.Scene)
    {
        this._sphere = this.CreateSphere(scene);
        this._position = this._sphere.position;
    }
    public CreateSphere(scene:BABYLON.Scene):BABYLON.Mesh
    {
        let Sphere:BABYLON.Mesh = BABYLON.MeshBuilder.CreateSphere("sphere", {diameter:2}, scene);
        return Sphere;
    }
    public get position(): BABYLON.Vector3 {
        return this._position;
    }
    public set position(value: BABYLON.Vector3) {
        this._position = value;
    }
    public get sphere(): BABYLON.Mesh {
        return this._sphere;
    }
    public set sphere(value: BABYLON.Mesh) {
            this._sphere = value;
    }
    

}

export class MultipleSphere extends Sphere
{
    constructor(scene:BABYLON.Scene)
    {
        super(scene);//create first sphere
        for (let index = 0; index < 5; index++)
        {
            let newSphere:BABYLON.Mesh = this.CreateSphere(scene);
            newSphere.position = new BABYLON.Vector3(0,index,0)
            
        }
    }
}



class Main
{
    public static Main():void
    {
        let renderCanvas:HTMLCanvasElement = document.getElementById("renderCanvas") as HTMLCanvasElement;
        let engine:BABYLON.Engine = new BABYLON.Engine(renderCanvas,true);

        let newScene:SceneOne = new SceneOne(engine,renderCanvas);
        
        
        //RUN RENDER LOOP
        engine.runRenderLoop(function()
        {
            newScene.scene.render();
        })

        console.log("Funciona perfectamente la herencía");
    }
}



Main.Main();
